@@ -485,7 +485,6 @@ build_cache (EBookBackendExchange *be)
 	e_book_backend_cache_set_populated (bepriv->cache);
 	bepriv->is_cache_ready=TRUE;
 	e_file_cache_thaw_changes (E_FILE_CACHE (bepriv->cache));
-
 	return NULL;
 }
 
@@ -1859,6 +1858,11 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 	switch (bepriv->mode) {
 
 	case GNOME_Evolution_Addressbook_MODE_LOCAL:
+		if (!bepriv->marked_for_offline) {
+			e_data_book_view_notify_complete (book_view, 
+					GNOME_Evolution_Addressbook_OfflineUnavailable);
+			return;
+		}
 		if (!bepriv->cache) {
 			e_data_book_view_notify_complete (book_view, 
 					GNOME_Evolution_Addressbook_Success);
@@ -1894,6 +1898,14 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 		
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
 
+		if (!be->priv->ctx) {
+			e_book_backend_notify_auth_required (backend);
+			e_data_book_view_notify_complete (book_view,
+						GNOME_Evolution_Addressbook_AuthenticationRequired);
+			bonobo_object_unref (book_view);
+			return;
+		}
+
 		/* execute the query */
 		rn = e_book_backend_exchange_build_restriction (query, 
 							bepriv->base_rn);
@@ -1902,7 +1914,7 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 					       field_names, n_field_names,
 					       rn, NULL, TRUE);
 		e2k_restriction_unref (rn);
-
+		
 		while ((result = e2k_result_iter_next (iter))) {
 			contact = e_contact_from_props (be, result);
 			if (contact) {
@@ -2129,12 +2141,13 @@ e_book_backend_exchange_authenticate_user (EBookBackend *backend,
 		account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
 		/* FIXME : Check for failures */
 		if (!exchange_account_get_context (account)) {
+			exchange_account_set_online (account);
 			if(!exchange_account_connect (account, password, &result)) {
 				e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_AuthenticationFailed);
 				return;
 			}
 		}
-		if (!bepriv->connected);
+		if (!bepriv->connected)
 			e_book_backend_exchange_connect (be);
 		if (e_book_backend_cache_is_populated (bepriv->cache)) {
 			if (bepriv->is_writable)
@@ -2259,8 +2272,6 @@ e_book_backend_exchange_load_source (EBookBackend *backend,
 		e_book_backend_set_is_writable (backend, FALSE);
 		e_book_backend_notify_writable (backend, FALSE);
 		e_book_backend_notify_connection_status (backend, FALSE);
-	}
-	if (bepriv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
 		if (!e_book_backend_cache_exists (bepriv->original_uri))
 			return GNOME_Evolution_Addressbook_OfflineUnavailable;
 	}
@@ -2271,7 +2282,10 @@ e_book_backend_exchange_load_source (EBookBackend *backend,
 		return GNOME_Evolution_Addressbook_Success;
 	}
 
+	// writable property will be set in authenticate_user callback
+	e_book_backend_set_is_writable (E_BOOK_BACKEND(backend), FALSE);
 	e_book_backend_set_is_loaded (E_BOOK_BACKEND (be), TRUE);
+	e_book_backend_notify_connection_status (E_BOOK_BACKEND (be), TRUE);
 
 	if (e_book_backend_cache_is_populated (bepriv->cache)) {
 		if (bepriv->is_writable)
@@ -2338,20 +2352,20 @@ e_book_backend_exchange_set_mode (EBookBackend *backend, int mode)
 
 	bepriv->mode = mode;
 	/* if (e_book_backend_is_loaded (backend)) { */
-		if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
-			e_book_backend_set_is_writable (backend, FALSE);
-			e_book_backend_notify_writable (backend, FALSE);
-			e_book_backend_notify_connection_status (backend, FALSE);
-			/* FIXME : free context ? */
-		} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
-			e_book_backend_set_is_writable (backend, bepriv->is_writable);
-			e_book_backend_notify_writable (backend, bepriv->is_writable);
-			e_book_backend_notify_connection_status (backend, TRUE);
-			account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
-			if (!exchange_account_get_context (account))
-				e_book_backend_notify_auth_required (backend);
-		}
-/*	} */
+	if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
+		e_book_backend_set_is_writable (backend, FALSE);
+		e_book_backend_notify_writable (backend, FALSE);
+		e_book_backend_notify_connection_status (backend, FALSE);
+		/* FIXME : free context ? */
+	} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
+		e_book_backend_set_is_writable (backend, bepriv->is_writable);
+		e_book_backend_notify_writable (backend, bepriv->is_writable);
+		e_book_backend_notify_connection_status (backend, TRUE);
+		account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+		if (!exchange_account_get_context (account))
+			e_book_backend_notify_auth_required (backend);
+	}
+	/*}*/
 }
 
 /**
@@ -2475,7 +2489,6 @@ e_book_backend_exchange_init (EBookBackendExchange *backend)
 	priv->cache		= NULL;
 	priv->original_uri 	= NULL;
 	priv->is_writable 	= TRUE;
-	priv->mode		= GNOME_Evolution_Addressbook_MODE_LOCAL;
 
 	priv->create_mutex      = g_mutex_new ();
 
